package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.enterprise.common.CountPage;
import cn.com.ty.lift.enterprise.oa.dto.AnnouncementRequest;
import cn.com.ty.lift.enterprise.oa.dto.AnnouncementResponse;
import cn.com.ty.lift.enterprise.oa.entity.Announcement;
import cn.com.ty.lift.enterprise.oa.service.AnnouncementService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * <p>
 * 公司公告 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */

@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/oa/announcement")
public class AnnouncementController {

    private AnnouncementService announcementService;

    /**
     * @description 根据id查找公告
     * @date 2019/12/6 9:24
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody AnnouncementRequest request) {
        Long id = request.getId();
        if(null == id || id <= 0){
            return RestResponse.failParam();
        }
        Announcement entity = announcementService.getById(id);
        return null != entity ? RestResponse.success(entity) : RestResponse.fail();
    }

    /**
     * @description 根据条件查找公告
     * @date 2019/12/6 9:24
     * @param
     * @return
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody AnnouncementRequest request){
        IPage<AnnouncementResponse> pages = announcementService.pageByCondition(request);
        if(null == pages){
            return RestResponse.fail();
        }
        long count = announcementService.count();
        CountPage<AnnouncementResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 添加公告
     * @date 2019/12/6 9:25
     * @param
     * @return
     */
    @PostMapping("add")
    public RestResponse add(@RequestBody Announcement entity){
        if(null == entity){
            return RestResponse.failParam();
        }
        boolean result = announcementService.save(entity);
        return result ? RestResponse.success(result) : RestResponse.fail();
    }

    /**
     * @description 根据id删除公告
     * @date 2019/12/6 9:25
     * @param
     * @return
     */
    @PostMapping("delete")
    public RestResponse delete(@RequestBody AnnouncementRequest request){
        Long id = request.getId();
        if(null == id || id <= 0){
            return RestResponse.failParam();
        }
        boolean result = announcementService.removeById(id);
        return result ? RestResponse.success(result) : RestResponse.fail();

    }
}
